@@ -12,6 +12,7 @@
 #include <X11/Xatom.h>
 
 #include "theme-common.h"
+#include "wm-common.h"
 #include "capplet-util.h"
 #include "eggcellrendererkeys.h"
 #include "activate-settings-daemon.h"
@@ -34,6 +35,7 @@ typedef struct
 
 const KeyListEntry desktop_key_list[] =
 {
+  { "/apps/panel/global/logout_key", ALWAYS_VISIBLE, 0 },
   { "/apps/panel/global/run_key", ALWAYS_VISIBLE, 0 },
   { "/apps/panel/global/menu_key", ALWAYS_VISIBLE, 0 },
   { "/apps/panel/global/screenshot_key", ALWAYS_VISIBLE, 0 },
@@ -46,11 +48,18 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/activate_window_menu",      ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_fullscreen",         ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_maximized",          ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/maximize",                  ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/unmaximize",                ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_shaded",             ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/close",                     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/minimize",                  ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/begin_move",                ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/begin_resize",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_on_all_workspaces",  N_WORKSPACES_GT, 1 },
+  /* These next three are out of order compared to the schemas file. */
+  { "/apps/metacity/window_keybindings/raise_or_lower",            ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/raise",                     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/lower",                     ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/move_to_workspace_1",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_2",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_3",       N_WORKSPACES_GT, 2 },
@@ -69,7 +78,8 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/move_to_workspace_down",    N_WORKSPACES_GT, 1 },
   { "/apps/metacity/global_keybindings/switch_windows",            ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_panels",             ALWAYS_VISIBLE,  0 },
-  { "/apps/metacity/global_keybindings/focus_previous_window",     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_windows",             ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_panels",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/show_desktop",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_1",     N_WORKSPACES_GT, 1 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_2",     N_WORKSPACES_GT, 1 },
@@ -101,154 +111,33 @@ typedef struct
 {
   char *gconf_key;
   guint keyval;
-  GdkModifierType mask;
+  EggVirtualModifierType mask;
   gboolean editable;
   GtkTreeModel *model;
   guint gconf_cnxn;
+  char *description;
 } KeyEntry;
 
-static void reload_key_entries (GladeXML *dialog);
+static void  reload_key_entries (gpointer                wm_name,
+                                 GladeXML               *dialog);
+static char* binding_name       (guint                   keyval,
+                                 EggVirtualModifierType  mask,
+                                 gboolean                translate);
 
-/* Our WM Window */
-static Window wm_window = None;
-
-static char *
-get_wm_name (void)
+static GtkTreeModel*
+get_real_model (GtkTreeView *tree_view)
 {
-  Atom utf8_string, atom, type;
-  int result;
-  char *retval;
-  int format;
-  gulong nitems;
-  gulong bytes_after;
-  guchar *val;
+  GtkTreeModel *model;
+  GtkTreeModel *submodel;
   
-  if (wm_window == None)
-      return NULL;
+  model = gtk_tree_view_get_model (tree_view);
+
+  if (model)
+    submodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+  else
+    submodel = NULL;
   
-  utf8_string = XInternAtom (GDK_DISPLAY (), "UTF8_STRING", False);
-  atom = XInternAtom (GDK_DISPLAY (), "_NET_WM_NAME", False);
-  
-  gdk_error_trap_push ();
-  
-  result = XGetWindowProperty (GDK_DISPLAY (),
-			       wm_window,
-			       atom,
-			       0, G_MAXLONG,
-			       False, utf8_string,
-			       &type, &format, &nitems,
-			       &bytes_after, (guchar **)&val);
-
-  if (gdk_error_trap_pop () || result != Success)
-    return NULL;
-
-  if (type != utf8_string ||
-      format != 8 ||
-      nitems == 0)
-    {
-      if (val)
-	XFree (val);
-      return NULL;
-    }
-
-  if (!g_utf8_validate (val, nitems, NULL))
-    {
-      XFree (val);
-      return NULL;
-    }
-
-  retval = g_strndup (val, nitems);
-
-  XFree (val);
-
-  return retval;
-}
-
-static gboolean
-is_metacity_running (void)
-{
-  char *wm_name;
-
-  wm_name = get_wm_name ();
-  
-  if (wm_name &&
-      strcmp (wm_name, "Metacity") == 0)
-    {
-      g_free (wm_name);
-      return TRUE;
-    }
-
-  g_free (wm_name);
-  return FALSE;
-}
-
-static void
-update_wm_window (void)
-{
-  Window *xwindow;
-  Atom type;
-  gint format;
-  gulong nitems;
-  gulong bytes_after;
-  
-  XGetWindowProperty (GDK_DISPLAY (), GDK_ROOT_WINDOW (),
-		      XInternAtom (GDK_DISPLAY (), "_NET_SUPPORTING_WM_CHECK", False),
-		      0, G_MAXLONG, False, XA_WINDOW, &type, &format,
-		      &nitems, &bytes_after, (guchar **) &xwindow);
-
-  if (type != XA_WINDOW)
-    {
-      wm_window = None;
-      return;
-    }
-
-  gdk_error_trap_push ();
-  XSelectInput (GDK_DISPLAY (), *xwindow, StructureNotifyMask | PropertyChangeMask);
-  XSync (GDK_DISPLAY (), False);
-  
-  if (gdk_error_trap_pop ())
-    {
-      XFree (xwindow);
-      wm_window = None;
-      return;
-    }
-
-  wm_window = *xwindow;
-  XFree (xwindow);
-}
-
-static GdkFilterReturn 
-wm_window_event_filter (GdkXEvent *xev,
-			GdkEvent  *event,
-			gpointer   data)
-{
-  XEvent *xevent = (XEvent *)xev;
-
-  if ((xevent->type == DestroyNotify &&
-       wm_window != None && xevent->xany.window == wm_window) ||
-      (xevent->type == PropertyNotify &&
-       xevent->xany.window == GDK_ROOT_WINDOW () &&
-       xevent->xproperty.atom == (XInternAtom (GDK_DISPLAY (),  "_NET_SUPPORTING_WM_CHECK", False))) ||
-      (xevent->type == PropertyNotify &&
-       wm_window != None && xevent->xany.window == wm_window &&
-       xevent->xproperty.atom == (XInternAtom (GDK_DISPLAY (), "_NET_WM_NAME", False))))
-    {
-      update_wm_window ();
-      reload_key_entries (data);
-    }
-  
-  return GDK_FILTER_CONTINUE;
-}
-
-static void
-initialize_wm_handling (GladeXML *dialog)
-{
-  gdk_window_add_filter (NULL, wm_window_event_filter, dialog);
-
-  update_wm_window ();
-  
-  XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);
-  XSync (GDK_DISPLAY (), False);
+  return submodel;
 }
 
 static void
@@ -277,8 +166,16 @@ make_key_theme_menu_item (const gchar *key_theme)
 {
   GtkWidget *retval;
 
-  retval = gtk_menu_item_new_with_label (key_theme);
-  g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup (key_theme), g_free);
+  if (!strcmp (key_theme, "Default"))
+    {
+      retval = gtk_menu_item_new_with_label ("GNOME Default");
+      g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup ("Default"), g_free);
+    }
+  else
+    {
+      retval = gtk_menu_item_new_with_label (key_theme);
+      g_object_set_data_full (G_OBJECT (retval), LABEL_DATA, g_strdup (key_theme), g_free);
+    }
   g_signal_connect (G_OBJECT (retval), "activate", G_CALLBACK (menu_item_activate), NULL);
   gtk_widget_show (retval);
 
@@ -297,11 +194,11 @@ create_dialog (void)
 
 static char*
 binding_name (guint            keyval,
-              GdkModifierType  mask,
+              EggVirtualModifierType  mask,
               gboolean         translate)
 {
   if (keyval != 0)
-    return gtk_accelerator_name (keyval, mask);
+    return egg_virtual_accelerator_name (keyval, mask);
   else
     return translate ? g_strdup (_("Disabled")) : g_strdup ("disabled");
 }
@@ -309,7 +206,7 @@ binding_name (guint            keyval,
 static gboolean
 binding_from_string (const char      *str,
                      guint           *accelerator_key,
-                     GdkModifierType *accelerator_mods)
+                     EggVirtualModifierType *accelerator_mods)
 {
   g_return_val_if_fail (accelerator_key != NULL, FALSE);
   
@@ -320,7 +217,7 @@ binding_from_string (const char      *str,
       return TRUE;
     }
 
-  gtk_accelerator_parse (str, accelerator_key, accelerator_mods);
+  egg_accelerator_parse_virtual (str, accelerator_key, accelerator_mods);
 
   if (*accelerator_key == 0)
     return FALSE;
@@ -404,19 +301,73 @@ keybinding_key_changed (GConfClient *client,
   gtk_tree_model_foreach (key_entry->model, keybinding_key_changed_foreach, key_entry);
 }
 
+static int
+keyentry_sort_func (GtkTreeModel *model,
+                    GtkTreeIter  *a,
+                    GtkTreeIter  *b,
+                    gpointer      user_data)
+{
+  KeyEntry *key_entry_a;
+  KeyEntry *key_entry_b;
+  char *name_a;
+  char *name_b;
+  int retval;
+
+  /* mmmmm, super-slow. */
+
+  key_entry_a = NULL;
+  gtk_tree_model_get (model, a,
+                      KEYENTRY_COLUMN, &key_entry_a,
+                      -1);  
+
+  key_entry_b = NULL;
+  gtk_tree_model_get (model, b,
+                      KEYENTRY_COLUMN, &key_entry_b,
+                      -1);
+
+  
+  if (key_entry_a != NULL)
+    name_a = binding_name (key_entry_a->keyval,
+                           key_entry_a->mask,
+                           TRUE);
+  else
+    name_a = NULL;
+
+  if (key_entry_b != NULL)
+    name_b = binding_name (key_entry_b->keyval,
+                           key_entry_b->mask,
+                           TRUE);
+  else
+    name_b = NULL;
+
+  if (name_a && name_b)
+    retval = g_utf8_collate (name_a, name_b);
+  else if (name_a)
+    retval = 1;
+  else if (name_b)
+    retval = -1;
+  else
+    retval = 0;
+
+  g_free (name_a);
+  g_free (name_b);
+
+  return retval;
+}
 
 static void
 clear_old_model (GladeXML  *dialog,
 		 GtkWidget *tree_view)
 {
   GtkTreeModel *model;
+  GtkTreeModel *sort_model;
   GtkTreeIter iter;
   KeyEntry *key_entry;
   gboolean valid;
   GConfClient *client;
 
   client = gconf_client_get_default ();
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+  model = get_real_model (GTK_TREE_VIEW (tree_view));
 
   if (model != NULL)
     {
@@ -433,6 +384,7 @@ clear_old_model (GladeXML  *dialog,
 	    {
 	      gconf_client_notify_remove (client, key_entry->gconf_cnxn);
 	      g_free (key_entry->gconf_key);
+              g_free (key_entry->description);
 	      g_free (key_entry);
 	    }
 	}
@@ -440,7 +392,20 @@ clear_old_model (GladeXML  *dialog,
     }
 
   model = (GtkTreeModel *) gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), model);
+  
+  sort_model = gtk_tree_model_sort_new_with_model (model);
+
+  /* N_COLUMNS is just a place to stick the extra sort function */
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
+                                   N_COLUMNS,
+                                   keyentry_sort_func,
+                                   NULL, NULL);
+  
+  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), sort_model);
+
+  g_object_unref (G_OBJECT (model));
+  g_object_unref (G_OBJECT (sort_model));
+  
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("actions_swindow")),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
   gtk_widget_set_usize (WID ("actions_swindow"), -1, -1);
@@ -488,7 +453,7 @@ append_keys_to_tree (GladeXML           *dialog,
   gint i, j;
 
   client = gconf_client_get_default ();
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
+  model = get_real_model (GTK_TREE_VIEW (WID ("shortcut_treeview")));
 
   i = 0;
   gtk_tree_model_foreach (model, count_rows_foreach, &i);
@@ -550,6 +515,7 @@ append_keys_to_tree (GladeXML           *dialog,
 						       key_entry, NULL, NULL);
       binding_from_string (key_value, &key_entry->keyval, &key_entry->mask);
       g_free (key_value);
+      key_entry->description = g_strdup (gconf_schema_get_short_desc (schema));
 
       if (i == MAX_ELEMENTS_BEFORE_SCROLLING)
 	{
@@ -565,7 +531,7 @@ append_keys_to_tree (GladeXML           *dialog,
       if (gconf_schema_get_short_desc (schema))
 	gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
 			    DESCRIPTION_COLUMN,
-                            gconf_schema_get_short_desc (schema),
+                            key_entry->description,
 			    KEYENTRY_COLUMN, key_entry,
 			    -1);
       else
@@ -579,25 +545,19 @@ append_keys_to_tree (GladeXML           *dialog,
     }
 
   if (i == 0)
-    {
       gtk_widget_hide (WID ("shortcuts_vbox"));
-      gtk_widget_hide (WID ("shortcuts_hbox"));
-    }
   else
-    {
       gtk_widget_show (WID ("shortcuts_vbox"));
-      gtk_widget_show (WID ("shortcuts_hbox"));
-    }
 }
 
 static void
-reload_key_entries (GladeXML *dialog)
+reload_key_entries (gpointer wm_name, GladeXML *dialog)
 {
   clear_old_model (dialog, WID ("shortcut_treeview"));
   
   append_keys_to_tree (dialog, _("Desktop"), desktop_key_list);
   
-  if (is_metacity_running ())
+  if (strcmp((char *) wm_name, WM_COMMON_METACITY) == 0)
     {
       append_keys_to_tree (dialog, _("Window Management"), metacity_key_list);
     }
@@ -609,7 +569,7 @@ key_entry_controlling_key_changed (GConfClient *client,
 				   GConfEntry  *entry,
 				   gpointer     user_data)
 {
-  reload_key_entries (user_data);
+  reload_key_entries (wm_common_get_current_window_manager(), user_data);
 }
 
 static void
@@ -654,11 +614,39 @@ key_theme_changed (GConfClient *client,
 }
 
 
+static gboolean
+cb_check_for_uniqueness (GtkTreeModel *model,
+			 GtkTreePath  *path,
+			 GtkTreeIter  *iter,
+			 gpointer      user_data)
+{
+  KeyEntry *key_entry;
+  KeyEntry *tmp_key_entry;
+
+  key_entry = (KeyEntry *)user_data;
+  gtk_tree_model_get (key_entry->model, iter,
+		      KEYENTRY_COLUMN, &tmp_key_entry,
+		      -1);
+
+  if (tmp_key_entry != NULL &&
+      key_entry->keyval == tmp_key_entry->keyval &&
+      key_entry->mask   == tmp_key_entry->mask &&
+      /* be sure we don't claim a key is a dup of itself */
+      strcmp (key_entry->gconf_key, tmp_key_entry->gconf_key) != 0)
+    {
+      key_entry->editable = FALSE;
+      key_entry->gconf_key = tmp_key_entry->gconf_key;
+      key_entry->description = tmp_key_entry->description;
+      return TRUE;
+    }
+  return FALSE;
+}
+
 static void
 accel_edited_callback (GtkCellRendererText *cell,
                        const char          *path_string,
                        guint                keyval,
-                       GdkModifierType      mask,
+                       EggVirtualModifierType mask,
 		       guint                keycode,
                        gpointer             data)
 {
@@ -666,11 +654,11 @@ accel_edited_callback (GtkCellRendererText *cell,
   GtkTreeModel *model;
   GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
   GtkTreeIter iter;
-  KeyEntry *key_entry;
+  KeyEntry *key_entry, tmp_key;
   GError *err = NULL;
   char *str;
 
-  model = gtk_tree_view_get_model (view);
+  model = get_real_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_model_get (model, &iter,
 		      KEYENTRY_COLUMN, &key_entry,
@@ -678,7 +666,48 @@ accel_edited_callback (GtkCellRendererText *cell,
 
   /* sanity check */
   if (key_entry == NULL)
-    return;
+    {
+      gtk_tree_path_free (path);
+      return;
+    }
+
+  tmp_key.model  = model;
+  tmp_key.keyval = keyval;
+  tmp_key.mask   = mask;
+  tmp_key.gconf_key = key_entry->gconf_key;
+  tmp_key.description = NULL;
+  tmp_key.editable = TRUE; /* kludge to stuff in a return flag */
+
+  if (keyval != 0) /* any number of keys can be disabled */
+  gtk_tree_model_foreach (model, cb_check_for_uniqueness, &tmp_key);
+
+  /* flag to see if the new accelerator was in use by something */
+  if (!tmp_key.editable)
+    {
+      GtkWidget *dialog;
+      char *name;
+
+      name = egg_virtual_accelerator_name (keyval, mask);
+      
+      dialog =
+        gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
+				       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+				       GTK_MESSAGE_WARNING,
+				       GTK_BUTTONS_OK,
+                                _("The shortcut \"%s\" is already used for:\n \"%s\"\n"),
+                                name,
+                                tmp_key.description ?
+                                tmp_key.description : tmp_key.gconf_key);
+      g_free (name);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      /* set it back to its previous value. */
+      egg_cell_renderer_keys_set_accelerator (EGG_CELL_RENDERER_KEYS (cell),
+					      key_entry->keyval, key_entry->mask);
+      gtk_tree_path_free (path);
+      return;
+    }
 
   str = binding_name (keyval, mask, FALSE);
 
@@ -735,10 +764,10 @@ theme_changed_func (gpointer  uri,
 
       menu_item = make_key_theme_menu_item (info->name);
       if (!strcmp (info->name, "Default"))
-	/* Put default first, always */
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+      /* Put default first, always */
+        gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
       else
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
     }
 
   gtk_widget_show (menu);
@@ -823,11 +852,10 @@ setup_dialog (GladeXML *dialog)
   GConfClient *client;
   GList *key_theme_list;
   GtkCellRenderer *renderer;
+  GtkTreeViewColumn *column;
   GtkWidget *widget;
   gboolean found_keys = FALSE;
   GList *list;
-  GdkPixbuf *icon_pixbuf;
-  gchar *filename;
 
   client = gconf_client_get_default ();
 
@@ -853,7 +881,6 @@ setup_dialog (GladeXML *dialog)
       gtk_dialog_run (GTK_DIALOG (msg_dialog));
       gtk_widget_destroy (msg_dialog);
 
-      gtk_widget_hide (WID ("shortcut_hbox"));
     }
   else
     {
@@ -871,23 +898,33 @@ setup_dialog (GladeXML *dialog)
 		    "button_press_event",
 		    G_CALLBACK (start_editing_cb), dialog),
 		    
-  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (WID ("shortcut_treeview")),
-					       -1,
-					       _("Action"),
-					       gtk_cell_renderer_text_new (),
-					       "text", DESCRIPTION_COLUMN,
-					       NULL);
+  column = gtk_tree_view_column_new_with_attributes (_("Action"),
+						     gtk_cell_renderer_text_new (),
+						     "text", DESCRIPTION_COLUMN,
+						     NULL);
+  gtk_tree_view_column_set_resizable (column, TRUE);
+
+  gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
+  gtk_tree_view_column_set_sort_column_id (column, DESCRIPTION_COLUMN);  
+
   renderer = (GtkCellRenderer *) g_object_new (EGG_TYPE_CELL_RENDERER_KEYS,
 					       "editable", TRUE,
+					       "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X,
 					       NULL);
+
   g_signal_connect (G_OBJECT (renderer),
 		    "keys_edited",
                     G_CALLBACK (accel_edited_callback),
                     WID ("shortcut_treeview"));
-  gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (WID ("shortcut_treeview")),
-					      -1, _("Shortcut"),
-					      renderer,
-					      accel_set_func, NULL, NULL);
+
+  column = gtk_tree_view_column_new_with_attributes (_("Shortcut"), renderer, NULL);
+  gtk_tree_view_column_set_cell_data_func (column, renderer, accel_set_func, NULL, NULL);
+  gtk_tree_view_column_set_resizable (column, TRUE);
+
+  gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
+  /* N_COLUMNS is just a place to stick the extra sort function */
+  gtk_tree_view_column_set_sort_column_id (column, N_COLUMNS); 
+
   gconf_client_add_dir (client, "/apps/gnome_keybinding_properties", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_notify_add (client,
@@ -896,15 +933,10 @@ setup_dialog (GladeXML *dialog)
 			   dialog, NULL, NULL);
 
   /* set up the dialog */
-  reload_key_entries (dialog);
+  reload_key_entries (wm_common_get_current_window_manager(), dialog);
 
   widget = WID ("gnome-keybinding-dialog");
-  filename = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, "keyboard-shortcut.png", TRUE, NULL);
-  icon_pixbuf = gdk_pixbuf_new_from_file ("keyboard-shortcut.png", NULL);
-  if (icon_pixbuf != NULL)
-    gtk_window_set_icon (GTK_WINDOW (widget), icon_pixbuf);
-  g_free (filename);
-  g_object_unref (icon_pixbuf);
+  capplet_set_icon (widget, "keyboard-shortcut.png");
   gtk_widget_show (widget);
 
   g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), NULL);
@@ -929,7 +961,7 @@ main (int argc, char *argv[])
   activate_settings_daemon ();
 
   dialog = create_dialog ();
-  initialize_wm_handling (dialog);
+  wm_common_register_window_manager_change ((GFunc)(reload_key_entries), dialog);
   setup_dialog (dialog);
   
   gtk_main ();
